def create_node_table():
    sql = """
        CREATE TABLE IF NOT EXISTS Nodes (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        type_id INTEGER NOT NULL,
        graph_id INTEGER NOT NULL,
        FOREIGN KEY (type_id) REFERENCES Types(id)
        );
        """
    return sql

def create_type_table():
    sql = """
        CREATE TABLE IF NOT EXISTS Types (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        type TEXT NOT NULL UNIQUE
        );
        """
    return sql

def create_graph_table():
    sql = """
        CREATE TABLE IF NOT EXISTS Graphs (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        action TEXT NOT NULL
        );
        """
    return sql

def get_type_index(type):
    sql = """
        SELECT id FROM Types
        WHERE type="{}";
        """.format(type)
    return sql

def insert_type(new_type):
    sql = """
        INSERT OR IGNORE INTO Types (type) VALUES ("{}");
        """.format(new_type)
    return sql

def create_edge_table():
    sql = """
        CREATE TABLE IF NOT EXISTS Edges (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        type_id INTEGER NOT NULL,
        src_node_id INTEGER NOT NULL,
        dst_node_id INTEGER NOT NULL,
        created_at INTEGER NOT NULL,
        graph_id INTEGER NOT NULL,
        FOREIGN KEY (type_id) REFERENCES Types(id),
        FOREIGN KEY (src_node_id) REFERENCES Nodes(id),
        FOREIGN KEY (dst_node_id) REFERENCES Nodes(id)
        );
        """
    return sql

def insert_node(type_id, graph_id):
    sql = """
        REPLACE INTO Nodes (type_id, graph_id) VALUES ({}, {});
        """.format(type_id, graph_id)
    return sql

def insert_edge(graph_id, edge_type_id, src_node_id, dst_node_id, created_at):
    sql = """
        REPLACE INTO Edges (type_id, src_node_id, dst_node_id, created_at, graph_id)
        VALUES ({}, {}, {}, {}, {});
        """.format(edge_type_id, src_node_id, dst_node_id,
            created_at, graph_id)
    return sql

def get_last_row_id():
    sql = """
        SELECT last_insert_rowid();
        """
    return sql

def insert_graph(action):
    sql = """
        INSERT INTO Graphs (action) VALUES ("{}");
        """.format(action)
    return sql

def get_next_graph_id():
    sql = """
        SELECT MAX(graph_id) AS g_id FROM Nodes
        """
    return sql



import paho.mqtt.client as mqtt
import sys
import os
import subprocess
import base64
import zlib
import sqlite3
import time
from termcolor import colored

class MQTTClient():
    def __init__(self):
        self.client = mqtt.Client()
        self.topics = []

    def publish(self, topic, msg):
        self.client.publish(topic, zlib.compress(msg))

    def subscribe(self, topic):
        if topic is not None:
            self.client.subscribe(topic, qos=0)
        else:
            print("MQTT topic improperly configured, exiting.")
            sys.exit()

    def on_connect(self, client, userdata, flags, rc):
        print("Connected with result code "+str(rc))

    def on_disconnect(self, client, userdata, rc=0):
        print("disconnected with result code "+ str(rc))
        self.client.loop_stop()

    def connect_client(self, user, passwd, host, port, on_message):
        print("Connecting MQTT subscriber...")
        self.client.on_connect = self.on_connect
        self.client.on_message = on_message
        self.client.on_disconnect = self.on_disconnect

        self.client.username_pw_set(user, passwd)
        self.client.connect(host, int(port), 60)
        self.client.loop_start()
        time.sleep(1)

    def disconnect_client(self):
        print("Stopping MQTT subscriber...")
        self.client.loop_stop()



import paho.mqtt.client as mqtt
import sys
import os
import subprocess
import base64
import zlib
import sqlite3
import time
from termcolor import colored

from . import config
from . import queries
from . import flake
from . import client

class Snowbank():
    def __init__(self, filter):
        DB_FILE = str(os.getcwd() + "/" + str(config.initFromConfig('DB_FILE')))
        if DB_FILE is not None:
            self.db_file = DB_FILE
            self.client = client.MQTTClient()
            db_conn = sqlite3.connect(self.db_file)
            cursor = db_conn.cursor()
            cursor.execute(queries.create_type_table())
            cursor.execute(queries.create_node_table())
            cursor.execute(queries.create_edge_table())
            cursor.execute(queries.create_graph_table())
            db_conn.commit()
            cursor.close()
        else:
            print("Could not find database file from configuration.")
            sys.exit()
        self.filter = filter
        self.flakes = {}

    def getFlake(self, id):
        return self.flakes[id]

    def collect_flake(self, action):
        db = sqlite3.connect(self.db_file)
        cursor = db.cursor()
        sql = queries.insert_graph(action)
        cursor.execute(sql)
        db.commit()
        cursor.execute(queries.get_last_row_id())
        id = cursor.fetchall()[0][0]
        currFlake = flake.Snowflake(id)
        self.client.client.user_data_set({'graph': currFlake})
        self.flakes[id] = currFlake
        return id

    def finalize(self, id):
        db_conn = sqlite3.connect(self.db_file)
        save = config.initFromConfig("SAVE_TO_DISK")
        if save == 'yes':
            if id in self.flakes:
                self.flakes[id].save_to_disk(self.db_file)
            else:
                print("ERROR: Flake not found")
        db_conn.commit()

    def on_message(self, client, userdata, msg):
        decoded_msg = zlib.decompress(base64.b64decode(msg.payload.decode('latin-1'))).decode('latin-1')
        self.filter.load_data(decoded_msg, userdata['graph'])

    def connect_client(self):
        topic = config.initFromConfig('MQTT_TOPIC')
        user = config.initFromConfig('MQTT_USERNAME')
        passwd = config.initFromConfig('MQTT_PASSWORD')
        host = config.initFromConfig('MQTT_HOST')
        port = config.initFromConfig('MQTT_PORT')
        camflow = config.initFromConfig('CAMFLOW_TOPIC')
        self.client.connect_client(user, passwd, host, port, self.on_message)
        self.client.subscribe(camflow)
        self.client.subscribe(topic)

    def disconnect_client(self):
        self.client.disconnect_client()




"""
The node class represents a single vertex in the network.
\param[in] type the assigned type to the node.
\param[in] index the node index which will serve to serialize it.
"""
class Node():
    def __init__(self, type, index):
        self.index = index
        # node type
        self.type = type
        # features of the node
        self.features = None

    def id(self):
        return self.index

    def getType(self):
        return self.type

    def hasFeatures(self):
        return self.features != None

    def getFeatures(self):
        return self.features

    def add_features(self, features):
        self.features = features
